package org.wit.myrent.activities;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.UUID;

import org.wit.android.helpers.ContactHelper;
import org.wit.myrent.R;
import org.wit.myrent.app.MyRentApp;
import org.wit.myrent.models.Portfolio;
import org.wit.myrent.models.Residence;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.support.v4.app.FragmentManager;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.app.DatePickerDialog;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.DatePicker;
import android.widget.EditText;
import static org.wit.android.helpers.IntentHelper.sendEmail;
import static org.wit.android.helpers.IntentHelper.navigateUp;
import static org.wit.android.helpers.CameraHelper.showPhoto;

import org.wit.android.helpers.MapHelper;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;

public class ResidenceFragment extends SupportMapFragment implements  TextWatcher, 
                                                                      OnCheckedChangeListener,
                                                                      OnClickListener, 
                                                                      DatePickerDialog.OnDateSetListener, 
                                                                      GoogleMap.OnMarkerDragListener,
                                                                      GoogleMap.OnCameraChangeListener,
                                                                      View.OnLongClickListener

{
  public static final String EXTRA_RESIDENCE_ID = "myrent.RESIDENCE_ID";
  private static final int REQUEST_CONTACT = 1;
  private static final int REQUEST_PHOTO = 0;

  private EditText geolocation;
  private CheckBox rented;
  private Button dateButton;
  private Button tenantButton;
  private Button reportButton;

  private Residence residence;
  private Portfolio portfolio;

  SupportMapFragment mapFragment;
  GoogleMap gmap;
  Marker marker;
  LatLng markerPosition;
  boolean markerDragged;

  private ImageView cameraButton;
  private ImageView photoView;

  @Override
  public void onCreate(Bundle savedInstanceState)
  {
    super.onCreate(savedInstanceState);
    setHasOptionsMenu(true);

    UUID resId = (UUID) getArguments().getSerializable(EXTRA_RESIDENCE_ID);

    MyRentApp app = (MyRentApp) getActivity().getApplication();
    portfolio = app.portfolio;
    residence = portfolio.getResidence(resId);
  }

  @Override
  public View onCreateView(LayoutInflater inflater, ViewGroup parent, Bundle savedInstanceState)
  {
    super.onCreateView(inflater, parent, savedInstanceState);
    View v = inflater.inflate(R.layout.fragment_residence, parent, false);

    getActivity().getActionBar().setDisplayHomeAsUpEnabled(true);
    addListeners(v);
    updateControls(residence);
    showPhoto(getActivity(), residence, photoView);
    return v;
  }

  private void addListeners(View v)
  {
    geolocation = (EditText) v.findViewById(R.id.geolocation);
    dateButton = (Button) v.findViewById(R.id.registration_date);
    rented = (CheckBox) v.findViewById(R.id.isrented);
    tenantButton = (Button) v.findViewById(R.id.tenant);
    reportButton = (Button) v.findViewById(R.id.residence_reportButton);
    cameraButton = (ImageView) v.findViewById(R.id.camera_button);
    photoView = (ImageView) v.findViewById(R.id.myrent_imageView);

    geolocation.addTextChangedListener(this);
    dateButton.setOnClickListener(this);
    rented.setOnCheckedChangeListener(this);
    tenantButton.setOnClickListener(this);
    reportButton.setOnClickListener(this);
    cameraButton.setOnClickListener(this);
    //photoView.setOnClickListener(this);
    photoView.setOnLongClickListener(this);
  }

  public void updateControls(Residence residence)
  {
    geolocation.setText(residence.geolocation);
    rented.setChecked(residence.rented);
    dateButton.setText(residence.getDateString());
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item)
  {
    switch (item.getItemId())
    {
    case android.R.id.home:
      navigateUp(getActivity());
      return true;
    default:
      return super.onOptionsItemSelected(item);
    }
  }

  @Override
  public void onPause()
  {
    super.onPause();
    portfolio.saveResidences();
  }

  @Override
  public void onActivityResult(int requestCode, int resultCode, Intent data)
  {
    if (resultCode != Activity.RESULT_OK)
    {
      return;
    }
    switch (requestCode)
    {
    case REQUEST_CONTACT:
      String name = ContactHelper.getContact(getActivity(), data);
      residence.tenant = name;
      tenantButton.setText(name);
      break;
    case REQUEST_PHOTO:
      String filename = data.getStringExtra(ResidenceCameraActivity.EXTRA_PHOTO_FILENAME);
      if (filename != null)
      {
        residence.photo = filename;
        showPhoto(getActivity(), residence, photoView );
      }
      break;
    }
  }
  /* ====================== textwatcher ============================================*/
  @Override
  public void beforeTextChanged(CharSequence s, int start, int count, int after)
  {}

  @Override
  public void onTextChanged(CharSequence s, int start, int before, int count)
  {}

  @Override
  public void afterTextChanged(Editable c)
  {
    String thisGeolocation = c.toString();
    Log.i(this.getClass().getSimpleName(), "geolocation " + thisGeolocation);
    residence.geolocation = thisGeolocation;
    getActivity().setTitle(thisGeolocation);
    // using a flag, markerDragged, to avoid race condition
    if (markerDragged == true)
    {
      markerDragged = false;
    }
    else
    {
      renderMap(MapHelper.latLng(getActivity(), thisGeolocation));
    }
  }
  /* ====================== cbeckbox ============================================*/
  @Override
  public void onCheckedChanged(CompoundButton buttonView, boolean isChecked)
  {
    residence.rented = isChecked;
  }
  /* ====================== onclick listeners ===================================*/
  @Override
  public void onClick(View v)
  {
    switch (v.getId())
    {
    case R.id.registration_date:
      Calendar c = Calendar.getInstance();
      DatePickerDialog dpd = new DatePickerDialog(getActivity(), this, c.get(Calendar.YEAR), c.get(Calendar.MONTH),
          c.get(Calendar.DAY_OF_MONTH));
      dpd.show();
      break;
    case R.id.tenant:
      Intent i = new Intent(Intent.ACTION_PICK, ContactsContract.Contacts.CONTENT_URI);
      startActivityForResult(i, REQUEST_CONTACT);
      if (residence.tenant != null)
      {
        tenantButton.setText("Tenant: " + residence.tenant);
      }
      break;
    case R.id.residence_reportButton:
      sendEmail(getActivity(), "", getString(R.string.residence_report_subject),
          residence.getResidenceReport(getActivity()));
      break;
    case R.id.camera_button:
      Intent ic = new Intent(getActivity(), ResidenceCameraActivity.class);
      startActivityForResult(ic, REQUEST_PHOTO);
      break;
    }
  }
  /* ====================== datepicker =====================================*/
  @Override
  public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth)
  {
    Date date = new GregorianCalendar(year, monthOfYear, dayOfMonth).getTime();
    residence.date = date;
    dateButton.setText(residence.getDateString());
  }

  /* ====================== map ========================================== */
  private void initializeMapFragment()
  {
    FragmentManager fm = getChildFragmentManager();
    mapFragment = (SupportMapFragment) fm.findFragmentById(R.id.map);
    if (mapFragment == null)
    {
      mapFragment = SupportMapFragment.newInstance();
      fm.beginTransaction().replace(R.id.map, mapFragment).commit();
    }
  }

  @Override
  public void onMarkerDragStart(Marker arg0)
  {}
  
  @Override
  public void onMarkerDrag(Marker arg0)
  {}

  @Override
  public void onMarkerDragEnd(Marker arg0)
  {
    residence.geolocation = MapHelper.latLng(arg0.getPosition());
    getActivity().setTitle(residence.geolocation);
    gmap.animateCamera(CameraUpdateFactory.newLatLng(arg0.getPosition()));
    markerDragged = true;
    geolocation.setText(residence.geolocation);
  }

  /*
   * this Camera refers to Google map, not device camera
   * When camera change equivalent to changed map position here
   * For example by panning map to new position
   * Method does the following:
   *      saves zoom position to model layer
   *      removes existing non-null marker
   *      creates and positions new marker to residence geolocation
   */
  @Override
  public void onCameraChange(CameraPosition arg0)
  {
    residence.zoom = arg0.zoom;
    markerPosition = MapHelper.latLng(getActivity(), residence.geolocation);
    if (marker != null)
    {
      marker.remove();
    }

    marker = gmap.addMarker(new MarkerOptions().position(markerPosition).draggable(true).title("Residence").alpha(0.7f)
        .snippet("GPS : " + markerPosition.toString()));

  }
  /*
   * initializes and renders the map
   * sets map type, example HYBRID in this case
   * sets the drag marker listener (but does not create a marker)
   * sets the map (camera) changed listener
   */
  private void renderMap(LatLng markerPosition)
  {
    if (mapFragment != null)
    {
      gmap = mapFragment.getMap();
      if (gmap != null)
      {
        gmap.animateCamera(CameraUpdateFactory.newLatLngZoom(markerPosition, (float) residence.zoom));
        gmap.setMapType(GoogleMap.MAP_TYPE_HYBRID);
        gmap.setOnMarkerDragListener(this);
        gmap.setOnCameraChangeListener(this);//this Camera refers to Google map, not device camera
      }
    }
  }

  /* @see http://stackoverflow.com/questions/18206615/how-to-use-google-map-v2-inside-fragment 
   * Necessary to wait until ResidenceFragment created before initializing map fragment
   * Note that map fragment is nested within residence fragment. See the xml layout file for residence frag
  */

  @Override
  public void onActivityCreated(Bundle savedInstanceState)
  {
    super.onActivityCreated(savedInstanceState);
    initializeMapFragment();
  }

  @Override
  public void onStart()
  {
    super.onStart();
    renderMap(MapHelper.latLng(getActivity(), residence.geolocation));
  }

  /* ====================== longpress thumbnail ===================================*/
  /*
   * Long press the bitmap image to view photo in single-photo gallery
   */
  @Override
  public boolean onLongClick(View v)
  {
    Intent i = new Intent(getActivity(), ResidenceGalleryActivity.class);
    i.putExtra(EXTRA_RESIDENCE_ID, residence.id);
    startActivity(i);
    return true;
  }
}